@@ -1,10 +1,14 @@
+from typing import Any
 from typing import Generator
-from typing import Iterable, Mapping
+from typing import Iterable
+from typing import List
+from typing import Mapping
 from typing import Optional
-from typing import Sized
+from typing import Tuple
 
 from _pytest._io.saferepr import saferepr
 from _pytest.assertion.util import _compare_eq_any
+from _pytest.config import Config
 
 
 class UnorderedList(list):
@@ -14,29 +18,26 @@ class UnorderedList(list):
                 "cannot make unordered comparisons to non-iterable: {!r}".format(expected)
             )
         if isinstance(expected, Mapping):
-            raise TypeError(
-                "cannot make unordered comparisons to mapping: {!r}".format(expected)
-            )
+            raise TypeError("cannot make unordered comparisons to mapping: {!r}".format(expected))
         super().__init__(expected)
         self._expected_type = type(expected) if check_type else None
 
-    def __eq__(self, actual: Iterable) -> bool:
+    def __eq__(self, actual: object) -> bool:
         if self._expected_type is not None and self._expected_type != type(actual):
             return False
         if not isinstance(actual, Iterable):
             return self.copy() == actual
-        if not isinstance(actual, Sized):
-            actual = list(actual)
-        if len(actual) != len(self):
+        actual_list = list(actual)
+        if len(actual_list) != len(self):
             return False
-        extra_left, extra_right = _compare_eq_unordered(self, actual)
+        extra_left, extra_right = _compare_eq_unordered(self, actual_list)
         return not extra_left and not extra_right
 
-    def __ne__(self, actual: Iterable) -> bool:
+    def __ne__(self, actual: object) -> bool:
         return not (self == actual)
 
 
-def unordered(*args, check_type: Optional[bool] = None) -> UnorderedList:
+def unordered(*args: Any, check_type: Optional[bool] = None) -> UnorderedList:
     if len(args) == 1:
         if check_type is None:
             check_type = not isinstance(args[0], Generator)
@@ -44,7 +45,7 @@ def unordered(*args, check_type: Optional[bool] = None) -> UnorderedList:
     return UnorderedList(args, check_type=False)
 
 
-def _compare_eq_unordered(left: Iterable, right: Iterable):
+def _compare_eq_unordered(left: Iterable, right: Iterable) -> Tuple[List, List]:
     extra_left = []
     extra_right = list(right)
     for elem in left:
@@ -55,7 +56,9 @@ def _compare_eq_unordered(left: Iterable, right: Iterable):
     return extra_left, extra_right
 
 
-def pytest_assertrepr_compare(config, op, left, right):
+def pytest_assertrepr_compare(
+    config: Config, op: str, left: Any, right: Any
+) -> Optional[List[str]]:
     if (isinstance(left, UnorderedList) or isinstance(right, UnorderedList)) and op == "==":
         verbose = config.getoption("verbose")
         left_repr = saferepr(left)
@@ -80,3 +83,4 @@ def pytest_assertrepr_compare(config, op, left, right):
                 for item in extra_right:
                     result.append(saferepr(item))
         return result
+    return None
